//
// Realizzazione del tipo di dato Vettore
// Implementazione della Classe vector
//

#include <math.h>
#include "Vector.H"


// Costruttore: crea un Vettore "vuoto"
vector::vector()
{
  for (int i = 0; i < SIZE; i++)
    c[i] = 0.0;
}


// Costruttore: crea un Vettore copiando gli elementi da un array
vector::vector(double *a)
{
  for (int i = 0; i < SIZE; i++)
    c[i] = a[i];
}


// Somma fra Vettori
vector operator+ (const vector& v, const vector& w)
{
  vector u;
  for (int i = 0; i < SIZE; i++)
    u.c[i] = v.c[i] + w.c[i];

  return(u);
}


// Sottrazione fra Vettori
vector operator- (const vector& v, const vector& w)
{
  vector u;
  for (int i = 0; i < SIZE; i++)
    u.c[i] = v.c[i] - w.c[i];

  return(u);
}


// Negazione di un Vettore
vector operator- (const vector& v)
{
  vector u;
  for (int i = 0; i < SIZE; i++)
    u.c[i] = -v.c[i];

  return(u);
}


// Moltiplicazione di uno scalare per un Vettore
vector operator* (double a, const vector& w)
{
  vector u;
  for (int i = 0; i < SIZE; i++)
    u.c[i] = a * w.c[i];

  return(u);
}


// Moltiplicazione di un Vettore per uno scalare
vector operator* (const vector& v ,double b)
{
  vector u;

  for (int i = 0; i < SIZE; i++)
    u.c[i] = v.c[i] * b;

  return(u);
}


// Prodotto Scalare
double operator* (const vector& v, const vector& w)
{
  double sum = 0.0;

  for (int i = 0; i < SIZE; i++)
    sum += v.c[i] * w.c[i];

  return(sum);
}


// Test di uguaglianza
int operator== (const vector& v, const vector& w)
{
  for (int i = 0; i < SIZE; i++)
    if( v.c[i] != w.c[i])		// Appena uno degli elementi differisce
      return 0;				// si ritorna "falso"

  return 1;
}


// Test per la non uguaglianza
int operator!= (const vector& v, const vector& w)
{
  for (int i = 0; i < SIZE; i++)
    if( v.c[i] != w.c[i])		// Appena uno degli elementi differisce
      return 1;				// si ritorna "vero"

  return 0;
}


// Somma ed assegnamento
vector& vector::operator+= (const vector& w)
{
  for (int i = 0; i < SIZE; i++)
    c[i] += w.c[i];

  return(*this);
}


// Sottrazione ed assegnamento
vector& vector::operator-= (const vector& w)
{
  for (int i = 0; i < SIZE; i++)
    c[i] -= w.c[i];

  return(*this);
}


// Moltiplicazione per uno scalare ed assegnamento
vector& vector::operator*= (double b)
{
  for (int i = 0; i < SIZE; i++)
    c[i] *= b;

  return(*this);
}


// Indicizzazione
double& vector::operator[] (int i)
{
  if( i < LOWER || i > UPPER)
    {
      cout << "Subscript out of range!" << endl;
      exit(1);
    }

  return(c[i - LOWER]);
}


// Conversione di tipo da Vettore a double
// Restituisce la norma di un Vettore.
vector::operator double() const
{
  double sum = 0.0;
  for (int i = 0; i < SIZE; i++)
    sum += c[i] * c[i];

  return(sqrt(sum));
}


// Input di un Vettore
istream& operator>> (istream& ci, vector& v)
{
  for (int i = 0; i < SIZE; i++)
    ci >> v.c[i];

  return(ci);
}


// output di un vettore
ostream& operator<< (ostream& co, const vector& v)
{
  co << "(" << v.c[0];
  for (int i = 1; i < SIZE; i++)
    co << "," << v.c[i];
  co << ")";

  return(co);
}
